# -*- coding: utf-8 -*-
import sqlite3
import datetime
from constants import ALERT_HISTORY_TIME


class AlertManager(object):

    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(AlertManager, cls).__new__(
                                cls, *args, **kwargs)
            cls._instance._init()
        return cls._instance

    def _init(self):

        self._conn = sqlite3.connect(':memory:')
        self.initialize_db()

    def initialize_db(self):

        c = self._conn.cursor()
        c.execute("""CREATE TABLE alerts(
            id INTEGER PRIMARY KEY,
            cluster_id TEXT,
            message TEXT,
            node_address TEXT,
            status TEXT,
            alert_type TEXT,
            delete_it INT,
            timestamp TIMESTAMP)""")
        self._conn.commit()

    def add_alert(self, cluster_id, message, node_address, status, alert_type, delete_it=0):

        c = self._conn.cursor()
        c.execute("""INSERT INTO alerts VALUES (null, ?, ?, ?, ?, ?, ?, ?)""",
            (cluster_id, message, node_address, status, alert_type, delete_it, datetime.datetime.now().strftime("%Y/%m/%d %H:%M:%S")))
        return c.lastrowid

    def get_alerts(self, cluster_id, last_id=0):

        c = self._conn.cursor()
        c.execute("SELECT id, cluster_id, message, status, alert_type, timestamp FROM alerts WHERE id > ? and cluster_id like ?", (last_id, cluster_id))
        return c.fetchall()

    def get_active_red_alert_count(self, cluster_id):

        c = self._conn.cursor()
        c.execute("SELECT COUNT(id) FROM alerts WHERE cluster_id like ? and status like ? and delete_it not like ?", (cluster_id, "red", "1"))
        return c.fetchone()[0]

    def get_last_alert(self, cluster_id):

        c = self._conn.cursor()
        c.execute("SELECT * FROM alerts WHERE cluster_id like ? ORDER BY id DESC LIMIT 1", (cluster_id,))
        result = c.fetchall()
        if result == []:
            return None
        else:
            return result[0]

    def delete_alerts(self, cluster_id):

        c = self._conn.cursor()
        c.execute("DELETE FROM alerts WHERE cluster_id like ?", (cluster_id,))

    def delete_alerts_specific_to_node(self, node_address):

        c = self._conn.cursor()
        c.execute("DELETE FROM alerts WHERE node_address like ?", (node_address,))

    def delete_old_alerts(self, cluster_id):

        timestamp = datetime.datetime.now() - datetime.timedelta(minutes=ALERT_HISTORY_TIME)
        timestamp = timestamp.strftime("%Y/%m/%d %H:%M:%S")
        last_row = self.get_last_alert(cluster_id)
        last_id = 1
        if last_row is not None:
            last_id = last_row[0]
        c = self._conn.cursor()
        c.execute(
            "DELETE FROM alerts WHERE timestamp < ? and cluster_id like ? and id < ? and delete_it = 1",
            (timestamp, cluster_id, last_id))

    def mark_it_to_delete(self, alert_id):

        c = self._conn.cursor()
        c.execute("UPDATE alerts set delete_it = 1 WHERE id = ?", (alert_id,))
